/***********************************
  (C) Copyright 1992-1993; dit/upm
   Distributed under the conditions stated in the
   TOPO General Public License (see file LICENSE)
 ***********************************
 $Log: preproc.h,v $
 * Revision 1.7  1993/03/29  18:09:10  lotos
 * add constructor verification/s
 *
 * Revision 1.6  1993/03/24  17:51:24  lotos
 * suppress some preprocessing (now in glad)
 *
 * Revision 1.5  1993/01/18  18:12:15  lotos
 * distribution issues
 *
 * Revision 1.4  1992/09/02  15:44:30  lotos
 * new option to remove externa annotations
 * new option to print and label the equations
 * new debugging options: save modified CAST
 * overall debugging and improvements
 *
 * Revision 1.3  92/01/14  15:24:50  lotos
 * distribution issues
 * 
 * Revision 1.2  91/04/11  18:25:44  lotos
 * code to transform value premisses into equations, removed
 * there may be external operations that yield a non-external sort
 * 
 * Revision 1.1  91/02/06  20:05:17  lotos
 * Initial revision
 * 
 ***********************************/

/***********************************************************************

     "preproc.h": cabecera de "preproc.o".

***********************************************************************/

#ifndef PREPROC_H
#define PREPROC_H


void AsigEtiqEc();
/* Asigna etiquetas a las ecuaciones. */

int VerifAnotTipo();
/* Verifica las anotaciones del tipo de datos. */

int CorrExpExtMIEc();
/* Corrige el AST si hay expresiones externas en alguna ecuacion. */
/* (Solo importa el miembro izquierdo). */

int CorrRepVarMIEc();
/* Corrige el AST si hay repeticion de variables en alguna ecuacion. */
/* (Solo importa el miembro izquierdo). */

int EnVarMIEc();
/* Enlaza variables en cada una de las ecuaciones. */
/* (Miembro derecho y premisas, con el miembro izquierdo). */

int VerifConsTipo();
/* Verifica la consistencia de los constructores del tipo de datos. */

#endif
